import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Date;
import java.util.Random;
import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.mail.MessagingException;
import javax.mail.internet.MimeBodyPart;
import org.apache.axiom.attachments.ConfigurableDataHandler;
import org.apache.axiom.om.OMException;

/*
 * This class provide the createMultiPartRequest method and other supporting methods which compose 
 * the HTTP request from the given payload string and file to be attached. 
 */
public class MIMEUtils {

	private static byte[] CRLF = { 13, 10 };

	private static String s_boundary = null;

	public static void write(OutputStream outStream, String payload,
			String attachmentFile) {
		try {
			s_boundary = getMimeBoundary();
			String charSetEncoding = "UTF-8";
			String payloadContentType = "text/xml; charset=UTF-8";
			String contentId = "0.urn:uuid:883CA9C91CE9C469D912223894056542";

			startWritingMime(outStream, s_boundary);

			javax.activation.DataHandler dh = new javax.activation.DataHandler(
					payload, "text/xml; charset=" + charSetEncoding);
			MimeBodyPart rootMimeBodyPart = new MimeBodyPart();
			rootMimeBodyPart.setDataHandler(dh);

			rootMimeBodyPart.addHeader("Content-Type",
					"application/xop+xml; charset=" + charSetEncoding
							+ "; type=\"" + payloadContentType + "\"");
			rootMimeBodyPart.addHeader("Content-Transfer-Encoding", "binary");
			rootMimeBodyPart.addHeader("Content-ID", "<" + contentId + ">");

			writeBodyPart(outStream, rootMimeBodyPart, s_boundary);

			String cid = "urn:uuid:EE69C2F1D5C814ADF71222389408640";
			javax.activation.DataHandler attachmentDh = new javax.activation.DataHandler(
					new FileDataSource(new File(attachmentFile)));
			writeBodyPart(outStream, createMimeBodyPart(cid, attachmentDh),
					s_boundary);

			finishWritingMime(outStream);
			outStream.flush();

		} catch (IOException e) {
			throw new OMException("Error while writing to the OutputStream.", e);
		} catch (MessagingException e) {
			throw new OMException("Problem writing Mime Parts.", e);
		}
	}

	public static String getBoundary() {
		return s_boundary;
	}

	private static String getMimeBoundary() {
		return ("MIMEBoundaryurn:uuid:" + getInitialUUID() + new Date()
				.getTime()).replace(':', '_');
	}

	public static byte[] createMultiPartRequest(String payLoad,
			String attachmentFile) {
		
		try {
			ByteArrayOutputStream bos = new ByteArrayOutputStream(8192);
			write(bos, payLoad, attachmentFile);

			return bos.toByteArray();

		} catch (Exception io) {
			System.out.println("IO Exception :: " + io.getLocalizedMessage());
		}
		return null;
	}

	/** @throws IOException This will write the boundary with CRLF */
	public static void startWritingMime(OutputStream outStream, String boundary)
			throws IOException {
		writeMimeBoundary(outStream, boundary);
		//outStream.write(CRLF);
	}

	/** @throws IOException This will write the boundary to output Stream */
	public static void writeMimeBoundary(OutputStream outStream, String boundary)
			throws IOException {
		// REVIEW: This conversion is hard-coded to UTF-8.
		// The complete solution is to respect the charset setting of the message.
		// However this may cause problems in BoundaryDelimittedStream and other
		// lower level classes.
		outStream.write(new byte[] { 45, 45 });
		outStream.write(boundary.getBytes("UTF-8"));
	}

	/**
	 * Writes a CRLF for the earlier boundary then the BodyPart data with headers followed by
	 * boundary. Writes only the boundary. No more CRLF's are written after that.
	 *
	 * @throws IOException
	 * @throws MessagingException
	 */
	public static void writeBodyPart(OutputStream outStream, MimeBodyPart part,
			String boundary) throws IOException, MessagingException {
		outStream.write(CRLF);
		part.writeTo(outStream);
		outStream.write(CRLF);
		writeMimeBoundary(outStream, boundary);
		outStream.flush();
	}

	public static MimeBodyPart createMimeBodyPart(String contentID,
			DataHandler dataHandler) throws MessagingException {
		String encoding = null;
		MimeBodyPart mimeBodyPart = new MimeBodyPart();
		mimeBodyPart.setDataHandler(dataHandler);
		mimeBodyPart.addHeader("Content-ID", "<" + contentID + ">");
		mimeBodyPart.addHeader("Content-Type", dataHandler.getContentType());
		if (dataHandler instanceof ConfigurableDataHandler) {
			ConfigurableDataHandler configurableDataHandler = (ConfigurableDataHandler) dataHandler;
			encoding = configurableDataHandler.getTransferEncoding();
		}
		if (encoding == null) {
			encoding = "binary";
		}
		mimeBodyPart.addHeader("Content-Transfer-Encoding", encoding);
		return mimeBodyPart;
	}

	private static String getInitialUUID() {
		Random myRand = null;
		if (myRand == null) {
			myRand = new Random();
		}

		long rand = myRand.nextLong();

		String sid;
		try {
			sid = InetAddress.getLocalHost().toString();
		} catch (UnknownHostException e) {
			sid = Thread.currentThread().getName();
		}

		StringBuffer sb = new StringBuffer();
		sb.append(sid);
		sb.append(":");
		sb.append(Long.toString(rand));

		MessageDigest md5 = null;
		try {
			md5 = MessageDigest.getInstance("MD5");
		} catch (NoSuchAlgorithmException e) {
			throw new UnsupportedOperationException(
					"Unable to obtain MD5 digest algorithm: " + e.toString(), e);
		}

		md5.update(sb.toString().getBytes());
		byte[] array = md5.digest();

		StringBuffer sb2 = new StringBuffer();
		for (int j = 0; j < array.length; ++j) {
			int b = array[j] & 0xFF;
			sb2.append(Integer.toHexString(b));
		}

		int begin = myRand.nextInt();
		if (begin < 0) {
			begin = begin * -1;
		}
		begin = begin % 8;

		return sb2.toString().substring(begin, begin + 18).toUpperCase();
	}

	/** @throws IOException This will write "--" to the end of last boundary */
	public static void finishWritingMime(OutputStream outStream)
			throws IOException {
		outStream.write(new byte[] { 45, 45 });
	}

}
